from contextlib import contextmanager
from django.contrib.auth.models import User
from django.core import mail
from django.core.urlresolvers import reverse
from django.test import TestCase
from django.test.utils import override_settings
import djohno
from djohno.views import DjohnoTestException
import socket
from mock import Mock, patch
from smtplib import SMTPConnectError


@contextmanager
def login_superuser(client):
    client.login(username='admin', password='password')
    yield
    client.logout()


class DjohnoViewTests(TestCase):
    def setUp(self):
        User.objects.create_superuser('admin',
                                      'foo@example.com',
                                      'password')

    def test_djohno_frame_403s_without_login(self):
        """
        Tests to ensure loading the root djohno view without
        authenticating results in a 403.
        """
        url = reverse('djohno_frame')
        response = self.client.get(url)
        self.assertEqual(response.status_code, 403)
        self.assertTemplateUsed(response, '403.html')

    def test_djohno_index_403s_without_login(self):
        """
        Tests to ensure loading the framed djohno view without
        authenticating results in a 403.
        """
        url = reverse('djohno_index')
        response = self.client.get(url)
        self.assertEqual(response.status_code, 403)
        self.assertTemplateUsed(response, '403.html')

    def test_djohno_403_403s_without_login(self):
        """
        Tests to ensure loading the framed djohno 403 test view
        without authenticating results in a 403.
        """
        url = reverse('djohno_403')
        response = self.client.get(url)
        self.assertEqual(response.status_code, 403)
        self.assertTemplateUsed(response, '403.html')

    def test_djohno_404_403s_without_login(self):
        """
        Tests to ensure loading the framed djohno 404 test view
        without authenticating results in a 403.
        """
        url = reverse('djohno_404')
        response = self.client.get(url)
        self.assertEqual(response.status_code, 403)
        self.assertTemplateUsed(response, '403.html')

    def test_djohno_500_403s_without_login(self):
        """
        Tests to ensure loading the framed djohno 500 test view
        without authenticating results in a 403.
        """
        url = reverse('djohno_500')
        response = self.client.get(url)
        self.assertEqual(response.status_code, 403)
        self.assertTemplateUsed(response, '403.html')

    def test_djohno_mail_403s_without_login(self):
        """
        Tests to ensure loading the framed djohno email test view
        without authenticating results in a 403.
        """
        url = reverse('djohno_email')
        response = self.client.get(url)
        self.assertEqual(response.status_code, 403)
        self.assertTemplateUsed(response, '403.html')

    def test_djohno_frame_with_login(self):
        """
        Tests to ensure loading the djohno frame view is successful,
        and renders a few specific strings.
        """
        with login_superuser(self.client):
            url = reverse('djohno_frame')
            response = self.client.get(url)
            self.assertEqual(response.status_code, 200)
            self.assertContains(response, 'Djohno: Home')
            self.assertContains(response,
                                'src="%s"' % reverse('djohno_index'))

    def test_djohno_403_frame_with_login(self):
        """
        Tests to ensure loading the djohno framed 403 view is
        successful, and renders a few specific strings.
        """
        with login_superuser(self.client):
            url = reverse('djohno_frame_403')
            response = self.client.get(url)
            self.assertEqual(response.status_code, 200)
            self.assertContains(response, 'Djohno: 403 Check')
            self.assertContains(response,
                                'src="%s"' % reverse('djohno_403'))

    def test_djohno_404_frame_with_login(self):
        """
        Tests to ensure loading the djohno framed 404 view is
        successful, and renders a few specific strings.
        """
        with login_superuser(self.client):
            url = reverse('djohno_frame_404')
            response = self.client.get(url)
            self.assertEqual(response.status_code, 200)
            self.assertContains(response, 'Djohno: 404 Check')
            self.assertContains(response,
                                'src="%s"' % reverse('djohno_404'))

    def test_djohno_500_frame_with_login(self):
        """
        Tests to ensure loading the djohno framed 500 view is
        successful, and renders a few specific strings.
        """
        with login_superuser(self.client):
            url = reverse('djohno_frame_500')
            response = self.client.get(url)
            self.assertEqual(response.status_code, 200)
            self.assertContains(response, 'Djohno: 500 Check')
            self.assertContains(response,
                                'src="%s"' % reverse('djohno_500'))

    def test_djohno_email_frame_with_login(self):
        """
        Tests to ensure loading the djohno framed email test view is
        successful, and renders a few specific strings.
        """
        with login_superuser(self.client):
            url = reverse('djohno_frame_email')
            response = self.client.get(url)
            self.assertEqual(response.status_code, 200)
            self.assertContains(response, 'Djohno: Email Check')
            self.assertContains(response,
                                'src="%s"' % reverse('djohno_email'))

    def test_djohno_index_with_login(self):
        """
        Tests to ensure loading the djohno index view is successful.
        """
        with login_superuser(self.client):
            url = reverse('djohno_index')
            response = self.client.get(url)
            self.assertEqual(response.status_code, 200)

    def test_djohno_403_with_login(self):
        """
        Tests to ensure loading the djohno 403 test view results in a
        403, and uses the expected template (this test may break if
        you've overriden handler403 with a function that doesn't
        render 403.html).
        """
        with login_superuser(self.client):
            url = reverse('djohno_403')
            response = self.client.get(url)
            self.assertEqual(response.status_code, 403)
            self.assertTemplateUsed(response, '403.html')

    def test_djohno_404_with_login(self):
        """
        Tests to ensure loading the djohno 404 test view results in a
        404, and uses the expected template (this test may break if
        you've overriden handler404 with a function that doesn't
        render 404.html).
        """
        with login_superuser(self.client):
            url = reverse('djohno_404')
            response = self.client.get(url)
            self.assertEqual(response.status_code, 404)
            self.assertTemplateUsed(response, '404.html')

    def test_djohno_500_with_login(self):
        """
        Tests to ensure loading the djohno 500 test view raises an
        exception.
        """
        with login_superuser(self.client):
            url = reverse('djohno_500')
            with self.assertRaises(DjohnoTestException):
                self.client.get(url)

    @override_settings(DEFAULT_FROM_EMAIL='Foobar <foo@bar.com>')
    def test_mail_view_complex_from_address(self):
        """
        Ensure the mail view correctly sends emails, and sends the
        expected text (we have a "pretty" from address).
        """
        with login_superuser(self.client):
            url = reverse('djohno_email')
            response = self.client.get(url)
            self.assertEqual(response.status_code, 200)
            self.assertTemplateUsed(response, 'djohno/email.html')
            self.assertEqual(len(mail.outbox), 1)
            sent = mail.outbox[0]
            self.assertEqual(sent.subject, 'djohno email test')
            self.assertTrue(sent.body.find('Congratulations') != -1)
            self.assertEqual(sent.body.find('It\'s probably a good'), -1)
            self.assertEqual(sent.body.find('\n\n\n'), -1)
            self.assertEqual(len(sent.to), 1)
            self.assertEqual(sent.to[0], 'foo@example.com')
            self.assertContains(response, "successfully sent")
            self.assertContains(response, "foo@example.com")
            self.assertContains(response, "Foobar &lt;foo@bar.com&gt;")

    @override_settings(DEFAULT_FROM_EMAIL='simple@bar.com')
    def test_mail_view_simple_from_address(self):
        """
        Ensure the mail view correctly sends emails, and sends the
        expected text (we don't have a "pretty" from address, so it
        should tell us about that).
        """
        with login_superuser(self.client):
            url = reverse('djohno_email')
            response = self.client.get(url)
            self.assertEqual(response.status_code, 200)
            self.assertTemplateUsed(response, 'djohno/email.html')
            self.assertEqual(len(mail.outbox), 1)
            sent = mail.outbox[0]
            self.assertEqual(sent.subject, 'djohno email test')
            self.assertTrue(sent.body.find('Congratulations') != -1)
            self.assertNotEqual(sent.body.find('It\'s probably a good'), -1)
            self.assertEqual(sent.body.find('\n\n\n'), -1)
            self.assertEqual(len(sent.to), 1)
            self.assertEqual(sent.to[0], 'foo@example.com')
            self.assertContains(response, "successfully sent")
            self.assertContains(response, "foo@example.com")
            self.assertContains(response, "simple@bar.com")

    @override_settings(DEFAULT_FROM_EMAIL='notanemail')
    def test_mail_view_invalid_from_address(self):
        """
        Ensure the mail view correctly detects invalid from emails.
        """
        with login_superuser(self.client):
            url = reverse('djohno_email')
            response = self.client.get(url)
            self.assertEqual(response.status_code, 200)
            self.assertTemplateUsed(response, 'djohno/bad_email.html')
            self.assertTemplateUsed(response,
                                    'djohno/_bad_email_invalid.html')
            self.assertEqual(len(mail.outbox), 0)
            self.assertContains(response, "no email was sent")
            self.assertContains(response, "notanemail")

    @override_settings(DEFAULT_FROM_EMAIL='webmaster@localhost')
    def test_mail_view_default_from_address(self):
        """
        Ensure the mail view correctly detects the DEFAULT_FROM_EMAIL
        settings not being overriden.
        """
        with login_superuser(self.client):
            url = reverse('djohno_email')
            response = self.client.get(url)
            self.assertEqual(response.status_code, 200)
            self.assertTemplateUsed(response, 'djohno/bad_email.html')
            self.assertTemplateUsed(response,
                                    'djohno/_bad_email_default.html')
            self.assertEqual(len(mail.outbox), 0)
            self.assertContains(response, "Your Name")
            self.assertContains(response, "you@example.com")

    def test_mail_view_smtp_failure(self):
        """
        Ensure the mail view correctly handles SMTP failures.
        """

        def fake_send_mail(subject, message,
                           from_email, recipient_list,
                           fail_silently=False,
                           auth_user=None, auth_password=None,
                           connection=None):
            raise SMTPConnectError(1337, "SMTP is too awesome")

        with login_superuser(self.client):
            url = reverse('djohno_email')
            with patch('djohno.views.send_mail',
                       Mock(side_effect=fake_send_mail)):
                response = self.client.get(url)
                self.assertEqual(response.status_code, 200)
                self.assertTemplateUsed(response, 'djohno/email.html')
                self.assertEqual(len(mail.outbox), 0)
                self.assertContains(response, "failed to send")
                self.assertContains(response,
                                    "(1337, &#39;SMTP is too awesome&#39;)")

    def test_mail_view_socket_failure(self):
        """
        Ensure the mail view correctly handles socket failures
        (probably fairly unlikely except in local development
        scenarios, when you are without an internet connection).
        """

        def fake_send_mail(subject, message,
                           from_email, recipient_list,
                           fail_silently=False,
                           auth_user=None, auth_password=None,
                           connection=None):
            raise socket.error(1337, 'Sockets are too awesome')

        with login_superuser(self.client):
            url = reverse('djohno_email')
            with patch('djohno.views.send_mail',
                       Mock(side_effect=fake_send_mail)):
                response = self.client.get(url)
                self.assertEqual(response.status_code, 200)
                self.assertTemplateUsed(response, 'djohno/email.html')
                self.assertEqual(len(mail.outbox), 0)
                self.assertContains(response, "failed to send")
                self.assertContains(response,
                                    "[Errno 1337] Sockets are too awesome")

    def test_djohno_version_frame_with_login(self):
        """
        Tests to ensure loading the djohno framed app versions view is
        successful, and renders a few specific strings.
        """
        with login_superuser(self.client):
            url = reverse('djohno_frame_versions')
            response = self.client.get(url)
            self.assertEqual(response.status_code, 200)
            self.assertContains(response, 'Djohno: Versions')
            self.assertContains(response,
                                'src="%s"' % reverse('djohno_versions'))

    def test_djohno_versions_with_login(self):
        """
        Tests to ensure loading the djohno versions view is
        successful.
        """
        with login_superuser(self.client):
            url = reverse('djohno_versions')
            response = self.client.get(url)
            self.assertEqual(response.status_code, 200)
            self.assertContains(response,
                                '<td>Djohno</td>',
                                html=True)
            self.assertContains(response,
                                '<td>%s</td>' % djohno.__version__,
                                html=True)
            self.assertContains(response,
                                '<td>Django</td>',
                                html=True)
            self.assertContains(response,
                                '<tr>',
                                count=5)   # 4 rows, 1 header



from django.conf.urls import patterns, url

from .views import (
    frame_view,
    frame_403_view,
    frame_404_view,
    frame_500_view,
    frame_email_view,
    frame_versions_view,
    index_view,
    test_403,
    test_404,
    test_500,
    test_email,
    versions
)

urlpatterns = patterns(
    '',
    url(r'^$', frame_view, name='djohno_frame'),
    url(r'^403/$', frame_403_view, name='djohno_frame_403'),
    url(r'^404/$', frame_404_view, name='djohno_frame_404'),
    url(r'^500/$', frame_500_view, name='djohno_frame_500'),
    url(r'^email/$', frame_email_view, name='djohno_frame_email'),
    url(r'^versions/$', frame_versions_view, name='djohno_frame_versions'),
    url(r'^framed/index/$', index_view, name='djohno_index'),
    url(r'^framed/403/$', test_403, name='djohno_403'),
    url(r'^framed/404/$', test_404, name='djohno_404'),
    url(r'^framed/500/$', test_500, name='djohno_500'),
    url(r'^framed/email/$', test_email, name='djohno_email'),
    url(r'^framed/versions/$', versions, name='djohno_versions'),
)



from django.conf import settings
from django.core.validators import validate_email
from email.utils import parseaddr
import sys


def is_pretty_from_address(input):
    name, email = parseaddr(input)
    validate_email(email)

    if name:
        return True
    else:
        return False


def _get_version_from_app(app):
    if hasattr(app, 'get_version'):
        if callable(app.get_version):
            return app.get_version()
        return app.get_version

    if hasattr(app, 'VERSION'):
        return app.VERSION

    if hasattr(app, '__version__'):
        return app.__version__

    return None


def get_app_versions():
    versions = {}

    for app in list(settings.INSTALLED_APPS) + ['django']:
        __import__(app)
        app = sys.modules[app]

        version = _get_version_from_app(app)

        if version is None:
            continue

        if isinstance(version, (list, tuple)):
            version = '.'.join(str(o) for o in version)

        name = app.__name__.split('.')[-1].replace('_', ' ').capitalize()
        versions[name] = version

    return versions



from django.core.exceptions import ValidationError
from django.test import TestCase
import djohno
from djohno.utils import (
    is_pretty_from_address,
    get_app_versions
)


class DjohnoUtilTests(TestCase):
    def test_is_pretty_from_address_fails_on_bare_address(self):
        """
        Ensure normal email addresses aren't parsed as being "pretty".
        """
        self.assertFalse(is_pretty_from_address('foo@bar.com'))

    def test_is_pretty_from_succeeds_on_pretty_address(self):
        """
        Ensure pretty addresses (e.g. Foo <foo@bar.com>) are parsed as
        being "pretty".
        """
        self.assertTrue(is_pretty_from_address('Foo <foo@bar.com>'))

    def test_is_pretty_from_raises_validation_error_on_bad_input(self):
        """
        Ensure invalid email addresses (e.g. "hello") raise
        ValidationError if given invalid inputs.
        """
        with self.assertRaises(ValidationError):
            self.assertTrue(is_pretty_from_address('hello'))

    def test_get_app_versions(self):
        versions = get_app_versions()
        self.assertEqual(versions['Djohno'], djohno.__version__)
        self.assertEqual(versions['Baz'], '0.4.2')
        self.assertEqual(versions['Moo'], '0.42')



from setuptools import setup, find_packages

import djohno

setup(
    name='djohno',
    version=djohno.__version__,
    description="A tiny Django app for checking over your error pages.",
    long_description=open('README.rst').read(),
    author='Dominic Rodger',
    author_email='internet@dominicrodger.com',
    url='http://github.com/dominicrodger/djohno',
    license='BSD',
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        "Django>=1.4",
    ],
    tests_require=(
        'django-setuptest==0.1.3',
        'mock==1.0.1',
    ),
    test_suite='setuptest.setuptest.SetupTestSuite',
)



from django.conf import settings
from django.core.exceptions import (
    PermissionDenied,
    ValidationError
)
from django.core.mail import send_mail
from django.core.urlresolvers import reverse_lazy
from django.http import Http404
from django.shortcuts import render
from django.template.loader import render_to_string
from django.views.generic import View, TemplateView
from djohno.utils import (
    is_pretty_from_address,
    get_app_versions
)
import socket
from smtplib import SMTPException
import sys


class BaseFrameView(View):
    def get(self, request, *args, **kwargs):
        if not request.user.is_superuser:
            raise PermissionDenied
        return render(request, 'djohno/frame.html',
                      {'frame_url': self.frame_url,
                       'title': self.title})


class FrameView(BaseFrameView):
    title = 'Djohno: Home'
    frame_url = reverse_lazy('djohno_index')
frame_view = FrameView.as_view()


class Frame403View(BaseFrameView):
    title = 'Djohno: 403 Check'
    frame_url = reverse_lazy('djohno_403')
frame_403_view = Frame403View.as_view()


class Frame404View(BaseFrameView):
    title = 'Djohno: 404 Check'
    frame_url = reverse_lazy('djohno_404')
frame_404_view = Frame404View.as_view()


class Frame500View(BaseFrameView):
    title = 'Djohno: 500 Check'
    frame_url = reverse_lazy('djohno_500')
frame_500_view = Frame500View.as_view()


class FrameEmailView(BaseFrameView):
    title = 'Djohno: Email Check'
    frame_url = reverse_lazy('djohno_email')
frame_email_view = FrameEmailView.as_view()


class FrameVersionsView(BaseFrameView):
    title = 'Djohno: Versions'
    frame_url = reverse_lazy('djohno_versions')
frame_versions_view = FrameVersionsView.as_view()


class IndexView(View):
    def get(self, request, *args, **kwargs):
        if not request.user.is_superuser:
            raise PermissionDenied
        return render(request, 'djohno/index.html')
index_view = IndexView.as_view()


class BaseExceptionView(View):
    exception = None

    def _get_exception(self):
        raise self.exception('Intentionally raised exception to test error '
                             'handling.')

    def get(self, request, *args, **kwargs):
        if not request.user.is_superuser:
            raise PermissionDenied
        self._get_exception()


class Test403View(BaseExceptionView):
    exception = PermissionDenied
test_403 = Test403View.as_view()


class Test404View(BaseExceptionView):
    exception = Http404
test_404 = Test404View.as_view()


class DjohnoTestException(Exception):
    pass


class Test500View(BaseExceptionView):
    exception = DjohnoTestException
test_500 = Test500View.as_view()


class TestEmailView(View):
    def get(self, request, *args, **kwargs):
        if not request.user.is_superuser:
            raise PermissionDenied

        from_address = settings.DEFAULT_FROM_EMAIL
        try:
            is_pretty = is_pretty_from_address(from_address)
        except ValidationError as e:
            return render(request, 'djohno/bad_email.html',
                          {'from_email': from_address})

        message = render_to_string('djohno/email_body.txt',
                                   {'pretty_from': is_pretty})

        error = None

        try:
            send_mail('djohno email test',
                      message,
                      from_address,
                      [request.user.email, ],
                      fail_silently=False)
            sent_successfully = True
        except SMTPException as e:
            sent_successfully = False
            error = e
        except socket.error as e:
            sent_successfully = False
            error = e

        return render(request, 'djohno/email.html',
                      {'email': request.user.email,
                       'from_email': from_address,
                       'sent_successfully': sent_successfully,
                       'error': error})
test_email = TestEmailView.as_view()


class VersionsView(TemplateView):
    template_name = 'djohno/versions.html'

    def get_context_data(self, **kwargs):
        return {'sys': '%d.%d.%d' % sys.version_info[:3],
                'versions': get_app_versions()}
versions = VersionsView.as_view()
